==> recime-bot-runtime-2017.11.110911.tgz: package/src/facebook.ts <==
import fs = require("fs");
import util = require("util");
import request = require("request");

import {Channel} from './channel';

export class Facebook extends Channel {
    constructor(params: any){
        super({
            homedir : params.homedir,
            uid : params.uid,
            platform : 'facebook',
            logger: params.logger
        });
    }

    sendSenderAction(sender, name){
        return new Promise((resolve)=>{
            const options ={
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                    access_token: process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                },
                method: 'POST',
                json: {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: sender
                    },
                    "sender_action": name
                }
            };
            request(options, (err, response, body)=>{
                resolve();
            });
        })

    }

    execute(body, parameters, handler){
        return new Promise((resolve, reject)=>{
            this.params.logger.incoming(body);
                
            body.entry.forEach((pageEntry)=>{
                const params = { 
                    pageEntry: pageEntry,
                    parameters: parameters,
                    handler: handler
                }
                this.process(params).then((data)=>{
                    resolve(data);
                },(err)=>{
                    reject(err);
                });
            });
        });
    }

    handleRequest(ev, parameters, cb){
        const senderID = ev.sender.id;
        
        return new Promise ((resolve, reject)=>{
            this.sendSenderAction(senderID, 'typing_on');

            const send = (data)=> {
                let options ={
                    sender: senderID,
                    data : data ,
                    accessToken : process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                };
                this.sendMessage(0, options).then (()=>{
                    resolve(data);
                }, (err)=>{
                    reject(err);
                });
            };
            cb(this.params.uid, ev, this.params.platform, (data)=>{
                return send(data);
            }, (err)=>{
                send({
                    text : err.message || err
                });
            });
        });
    }

    sendEachMessage(body){
        return  new Promise ((resolve, reject)=>{
            const options = {
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                access_token: body.accessToken
                },
                method: 'POST',
                json:  {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: body.sender
                    },
                    message : {}
                }
            };

            options.json.message = body.data;

            const data = body.data;
          
            request(options, (err, response, result)=>{
                if (!err && response.statusCode === 200){
                    this.params.logger.outgoing(options, response.body);
                    resolve();
                }
                else{
                    const message = {
                        fb: JSON.stringify(result.error),
                        message: response.statusMessage,
                        code: response.statusCode
                    };
                    reject(message);
                }
            });
        });
    }

    sendMessage(index, body: any){
        return new Promise((resolve, reject)=>{
            if (Array.isArray(body.data)){
                let data = body.data[index];
                let item = {
                    sender : body.sender,
                    accessToken  : body.accessToken,
                    data : data
                };
                if (index < body.data.length){
                    this.sendEachMessage(item).then(()=>{
                        this.sendMessage(++index, body).then(()=>{
                            resolve();
                        }, (err)=>{
                            reject();
                        })
                    }, (err)=>{
                        reject();
                    });
                } else {
                    resolve();
                }
            }
            else {
                this.sendEachMessage(body).then (()=>{
                    resolve();
                }, (err)=> {
                     reject(err);
                });
            }
        });
    }

    process(options){
        return new Promise((resolve, reject)=>{
            const pageEntry = options.pageEntry;
            const pageID = pageEntry.id;
            const parameters = options.parameters;
               
            pageEntry.messaging.forEach((ev)=>{
                if(ev.message && !ev.message.is_echo){
                    this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                    .catch(reject)
                } else if(ev.postback){
                    this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                    .catch(reject)
                } else{
                    return resolve();
                }
            });
        });
    }

    verifyToken(query){
        return new Promise((resolve, reject)=>{
            const token = query['hub.verify_token'];
            if (this.params.uid === token){
                resolve(query["hub.challenge"])
            } else {
                reject(403)
            }
        });
    }
}

==> recime-bot-runtime-2017.11.131109.tgz: package/src/facebook.ts <==
import fs = require("fs");
import util = require("util");
import request = require("request");

import {Channel} from './channel';

export class Facebook extends Channel {
    constructor(params: any){
        super({
            homedir : params.homedir,
            uid : params.uid,
            platform : 'facebook',
            logger: params.logger
        });
    }

    sendSenderAction(sender, name){
        return new Promise((resolve)=>{
            const options ={
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                    access_token: process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                },
                method: 'POST',
                json: {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: sender
                    },
                    "sender_action": name
                }
            };
            request(options, (err, response, body)=>{
                resolve();
            });
        });
    }

    execute(body, parameters, handler){
        return new Promise((resolve, reject)=>{
            this.params.logger.incoming(body);
                
            body.entry.forEach((pageEntry)=>{
                const params = { 
                    pageEntry: pageEntry,
                    parameters: parameters,
                    handler: handler
                }
                this.process(params).then((data)=>{
                    resolve(data);
                },(err)=>{
                    reject(err);
                });
            });
        });
    }

    handleRequest(ev, parameters, cb){
        const senderID = ev.sender.id;
        
        return new Promise ((resolve, reject)=>{
            this.sendSenderAction(senderID, 'typing_on').then (()=>{
                const send = (data) => {
                    let options = {
                        sender: senderID,
                        data: data,
                        accessToken: process.env.FACEBOOK_PAGE_ACCESS_TOKEN || process.env.RECIME_FACEBOOK_ACCESS_TOKEN
                    };
                    this.sendMessage(0, options).then(() => {
                        resolve(data);
                    }, (err) => {
                        reject(err);
                    });
                };
                cb(this.params.uid, ev, this.params.platform, (data) => {
                    return send(data);
                }, (err) => {
                    send({
                        text: err.message || err
                    });
                });
            });
        });
    }

    sendEachMessage(body){
        return  new Promise ((resolve, reject)=>{
            const options = {
                uri: 'https://graph.facebook.com/v2.6/me/messages',
                qs: {
                access_token: body.accessToken
                },
                method: 'POST',
                json:  {
                    messaging_type: "RESPONSE",
                    recipient: {
                        id: body.sender
                    },
                    message : {}
                }
            };

            options.json.message = body.data;

            const data = body.data;
          
            request(options, (err, response, result)=>{
                if (!err && response.statusCode === 200){
                    this.params.logger.outgoing(options, response.body);
                    resolve();
                }
                else{
                    const message = {
                        fb: JSON.stringify(result.error),
                        message: response.statusMessage,
                        code: response.statusCode
                    };
                    reject(message);
                }
            });
        });
    }

    sendMessage(index, body: any){
        return new Promise((resolve, reject)=>{
            if (Array.isArray(body.data)){
                let data = body.data[index];
                let item = {
                    sender : body.sender,
                    accessToken  : body.accessToken,
                    data : data
                };
                if (index < body.data.length){
                    this.sendEachMessage(item).then(()=>{
                        this.sendMessage(++index, body).then(()=>{
                            resolve();
                        }, (err)=>{
                            reject();
                        })
                    }, (err)=>{
                        reject();
                    });
                } else {
                    resolve();
                }
            }
            else {
                this.sendEachMessage(body).then (()=>{
                    resolve();
                }, (err)=> {
                     reject(err);
                });
            }
        });
    }

    process(options){
        return new Promise((resolve, reject)=>{
            const pageEntry = options.pageEntry;
            const pageID = pageEntry.id;
            const parameters = options.parameters;
               
            pageEntry.messaging.forEach((ev)=>{
                if(ev.message && !ev.message.is_echo){
                    this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                    .catch(reject)
                } else if(ev.postback){
                    this.handleRequest(ev, parameters, options.handler)
                        .then(resolve)
                    .catch(reject)
                } else{
                    return resolve();
                }
            });
        });
    }

    verifyToken(query){
        return new Promise((resolve, reject)=>{
            const token = query['hub.verify_token'];
            if (this.params.uid === token){
                resolve(query["hub.challenge"])
            } else {
                reject(403)
            }
        });
    }
}
